/* dcf77.h -- DCF77 Time module
   Copyright 2002 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/param.h>
#include <gel/dcf77.h>
#include <gel/timer.h>

static struct timer dcf77_timer;
static unsigned char dcf77_bit_num;
static struct tm *dcf77_time;
static struct tm *dcf77_valid_time;
static struct tm dcf77_time_table[3];
static unsigned char dcf77_parity;
static unsigned char dcf77_is_synchronized;

/*! Event to report changes in synchronisation state.  */
event_type dcf77_sync;

/*! Event to notify new date available.  */
event_type dcf77_update_time;

/*
Mark number(s) Encodes (01.s=0, 0.2s=1)
0              Minute, always 0 (0.1s)

1-14           Reserved

15             0=Normal antenna, 1=backup antenna

16             1=Approaching change from MEZ to MESZ or back

17,18          Time zone 0,1=MEZ; 1,0=MESZ

19             The leap second is encoded in this bit one hour prior to
               occurrence.

20             Start bit for encoded time, always 1

21-27          1, 2, 4, 8, 10, 20, 40 Minutes (mark 21=1 minute)

28             P1 maintains even parity for marks 21-28

29-34          1,2,4,8,10,20 Hours (mark 29=1 hour)

35             P2 maintains even parity for marks 29-35

36-41          Day in month (1, 2, 4, 8, 10, 20)

42-44          Day in week (1,2,4)
45-49          Month number (1, 2, 4, 8, 10)

50-57          Year (1, 2, 4, 8, 10, 20, 40, 80)

58             P3 maintains even parity for marks 36-58 There is no mark
               transmitted for the 59th second.

     */

static const unsigned char dcf77_min_table[] = {
  2,
  4,
  8,
  10,
  20,
  40,
  0
};

static const unsigned char dcf77_hour_table[] = {
  2,
  4,
  8,
  10,
  20,
  0
};

static const unsigned char dcf77_day_table[] = {
  2,
  4,
  8,
  10,
  20
};

static const unsigned char dcf77_wday_table[] = {
  1,
  2,
  4
};

static const unsigned char dcf77_mon_table[] = {
  1,
  2,
  4,
  8,
  10
};

static const unsigned char dcf77_year_table[] = {
  1,
  2,
  4,
  8,
  10,
  20,
  40,
  80
};

static void
dcf77_save_time (void)
{
  dcf77_valid_time = dcf77_time;
  dcf77_time++;
  if (dcf77_time == &dcf77_time_table[3])
    dcf77_time = dcf77_time_table;
}

static int
dcf77_consume_bit (unsigned char value)
{
  unsigned char add;
  
  switch (dcf77_bit_num)
    {
      /* 0              Minute, always 0 (0.1s) */
    case 0:
      if (value != 0)
	return 0;
      break;

      /* 1-14           Reserved */
    case 1 ... 14:
      /* 15             0=Normal antenna, 1=backup antenna */
    case 15:
      /* 16             1=Approaching change from MEZ to MESZ or back */
    case 16:
      /* 17,18          Time zone 0,1=MEZ; 1,0=MESZ */
    case 17:
    case 18:
      /* 19             The leap second is encoded in this bit one
			hour prior to occurrence. */
    case 19:
      break;
      
      /* 20             Start bit for encoded time, always 1 */
    case 20:
      if (value != 1)
	return 0;
      break;

    case 21:
      dcf77_time->tm_min = value;
      dcf77_parity = value;
      break;

      /* 21-27          1, 2, 4, 8, 10, 20, 40 Minutes (mark 21=1 minute) */
      /* 28             P1 maintains even parity for marks 21-28 */
    case 22 ... 28:
      add = dcf77_min_table[dcf77_bit_num - 22];
      if (add && value)
	{
	  dcf77_time->tm_min += add;
	  dcf77_parity = ~dcf77_parity;
	}
      else if (add == 0 && dcf77_parity != value)
	return 1;
      break;


      /* 29-34          1,2,4,8,10,20 Hours (mark 29=1 hour) */
    case 29:
      dcf77_time->tm_hour = value;
      dcf77_parity = value;
      break;

      /* 35             P2 maintains even parity for marks 29-35 */
    case 30 ... 35:
      add = dcf77_hour_table[dcf77_bit_num - 30];
      if (add && value)
	{
	  dcf77_time->tm_hour += add;
	  dcf77_parity = ~dcf77_parity;
	}
      else if (add == 0 && dcf77_parity != value)
	return 1;
      break;

      /* 36-41          Day in month (1, 2, 4, 8, 10, 20) */
    case 36:
      dcf77_time->tm_wday = 0;
      dcf77_time->tm_mday = value;
      dcf77_parity = value;
      break;

    case 37 ... 41:
      if (value)
	{
	  add = dcf77_day_table[dcf77_bit_num - 37];
	  dcf77_time->tm_mday += add;
	  dcf77_parity = ~dcf77_parity;
	}
      break;

      /* 42-44          Day in week (1,2,4) */
    case 42 ... 44:
      if (value)
	{
	  add = dcf77_wday_table[dcf77_bit_num - 42];
	  dcf77_time->tm_wday += add;
	  dcf77_parity = ~dcf77_parity;
	}
      break;

      /* 45-49          Month number (1, 2, 4, 8, 10) */
    case 45 ... 49:
      if (value)
	{
	  add = dcf77_mon_table[dcf77_bit_num - 45];
	  dcf77_time->tm_mon += add;
	  dcf77_parity = ~dcf77_parity;
	}
      break;
      

      /* 50-57          Year (1, 2, 4, 8, 10, 20, 40, 80) */
    case 50 ... 57:
      if (value)
	{
	  add = dcf77_year_table[dcf77_bit_num - 50];
	  dcf77_time->tm_year += add;
	  dcf77_parity = ~dcf77_parity;
	}
      break;

      /* 58             P3 maintains even parity for marks 36-58
		There is no mark transmitted for the 59th second. */
    case 58:

      break;

    case 59:
      dcf77_save_time ();
      return 0;
      break;
    }
  dcf77_bit_num++;
}


static void
dcf77_timer_handler (struct timer *t)
{
  unsigned char value;

  value = dcf77_input_probe ();
  if (dcf77_is_synchronized == 0)
    {
      ;
    }
  
}


/*! Initialize the DCF77 module.

    The DCF77 module must be initialized to register a timer handler
    to poll and analyze the DCF77 serial input.

*/
void dcf77_initialize (void)
{
  dcf77_time = &dcf77_time_table[0];
  dcf77_valid_time = 0;
}


/*! Get the DCF77 UTC time.

    Return in the \b tm structure the current time (UTC).

    @param tp	Pointer to the tm structure holding the result
*/
const struct tm *dcf77_gettime (void)
{
  return dcf77_valid_time;
}


