/* panel-test.c -- Test of panel interface
   Copyright 2001 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of EBCS.

EBCS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

EBCS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with EBCS; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/*! @page lcd-test Test program for the LCD operations.

  This program tests the LCD operation by displaying patterns on
  the LCD panel.

  @htmlonly
  Source file: <a href="lcd-test_c-source.html">lcd-test.c</a>
  @endhtmlonly

 */

#include <sys/param.h>
#include <sys/locks.h>
#include <sys/sio.h>
#include <lcd.h>
#include <sys/spi.h>
#include <time.h>
#include <test.h>
#include <stdio.h>
#include <string.h>
#include <benchs.h>
#include <gel/panel.h>

#define NR_PANELS 64

/* Forward declarations.  */
int test_panel_get_id (struct panel *p);
void test_panel_enter (struct panel *p);
void test_panel_exit (struct panel *p);
int test_create_panels (void);
int test_panel_putchar (void);
int test_panel_putstring (void);
int test_get_active (void);
int test_panel_pushpop (void);
void panel_benchs (void);

struct panel root_panel;
struct panel panels[NR_PANELS];
long panel_table[NR_PANELS];

void
lcd_home()
{
  ;
}
void
lcd_goto (lcd_col_t col, lcd_line_t lin)
{
  ;
}

void
lcd_putchar (unsigned char c)
{
  ;
}

void
button_enable ()
{
  ;
}
void
button_disable ()
{
  ;
}



int
test_panel_get_id (struct panel *p)
{
  size_t pos;

  pos = (size_t) (p - panels);
  return pos;
}

void
test_panel_enter (struct panel *p)
{
  int id;

  id = test_panel_get_id (p);
  panel_table[id]++;
}

void
test_panel_exit (struct panel *p)
{
  int id;

  id = test_panel_get_id (p);
  if (panel_table[id] <= 0)
    test_error ("Invalid enter count for %ld", (long) id);
  panel_table[id]--;
}

int
test_create_panels ()
{
  int i;

  for (i = 0; i < NR_PANELS; i++)
    {
      struct panel *p;

      p = &panels[i];
      memset (p, 0, sizeof (p[0]));
      p->to_enter = test_panel_enter;
      p->to_exit = test_panel_exit;
      panel_create (p);

      panel_table[i] = 0;
    }
  return 0;
}

int
test_panel_putchar ()
{
  int i;
  lcd_col_t col;
  lcd_line_t line;
  
  for (i = 0; i < NR_PANELS; i++)
    {
      struct panel *p;

      p = &panels[i];
      for (col = 0; col < LCD_MAX_COLS; col++)
        for (line = 0; line < LCD_MAX_LINES; line++)
          panel_putchar (p, col, line, rand () & 0x0ff);
    }
  return 0;
}

int
test_panel_putstring ()
{
  int i, j;
  lcd_col_t col;
  lcd_line_t line;
  
  for (i = 0; i < NR_PANELS; i++)
    {
      struct panel *p;
      char str[8];

      for (j = 0; j < 8; j += 3)
        {
          unsigned short c = rand ();
          
          str[j] = c;
          str[j+1] = c >> 8;
          str[j+2] = str[j] ^ str[j+1];
        }
      str[7] = 0;
      
      p = &panels[i];
      for (col = 0; col < LCD_MAX_COLS; col += 3)
        for (line = 0; line < LCD_MAX_LINES; line++)
          panel_putstring (p, col, line, str);
    }
  return 0;
}

int
test_get_active ()
{
  int i;
  int active = -1;

  for (i = 0; i < NR_PANELS; i++)
    {
      if (panel_table[i] == 0)
        continue;

      if (panel_table[i] != 1)
        test_error ("Panel enter count for %ld is %ld",
                    (long) i, (long) panel_table[i]);

      if (active != -1)
        test_error ("Several active panels at the same time");

      active = i;
    }
  return active;
}

int
test_panel_pushpop ()
{
  int i, j;
  
  for (i = 0; i < NR_PANELS; i++)
    {
      struct panel *p = &panels[i];
      unsigned char c;
      int active;

      panel_push (p);
      panel_putchar (p, 0, 0, i + ' ');
      panel_refresh ();

      active = test_get_active ();
      if (active != i)
        return test_error ("Invalid active panel %ld (expect %ld)",
                           (long) active, (long) i);

      c = panel_getchar_at (0, 0, 0);
      if (c != i + ' ')
        return test_error ("Panel putchar/getchar failed");

      /* Pop everything and verify the popped character.  */
      for (j = i - 1; j >= 0; j--)
        {
          panel_pop ();
          c = panel_getchar_at (0, 0, 0);
          if (c != j + ' ')
            return test_error ("Panel pop failed to restore '%ld', got '%ld'",
                               (long) (j + ' '), (long) c);

          active = test_get_active ();
          if (active != j)
            return test_error ("Invalid active panel %ld (expect %ld)",
                               (long) active, (long) j);
        }

      /* Restore the panel stack.  */
      for (j = 1; j <= i; j++)
        panel_push (&panels[j]);
    }
  for (i = 0; i < NR_PANELS; i++)
    panel_pop ();

  return 0;
}

void
panel_benchs ()
{
  bench_t b;
  struct panel *p = &panels[0];
  
  bench_init (&b);

  panel_putchar (p, 0, 0, 'a');
  bench_start (&b);
  panel_putchar (p, 0, 0, ' ');
  bench_stop (&b);
  bench_report (&b, "panel_putchar (1)");

  bench_start (&b);
  panel_putchar (p, 0, 0, ' ');
  bench_stop (&b);
  bench_report (&b, "panel_putchar (2)");

  panel_putstring (p, 0, 0, "123456789abcdef0");
  bench_start (&b);
  panel_putstring (p, 0, 0, "0123456789abcdef");
  bench_stop (&b);
  bench_report (&b, "panel_putstring 16 ch (1)");

  bench_start (&b);
  panel_putstring (p, 0, 0, "0123456789abcdef");
  bench_stop (&b);
  bench_report (&b, "panel_putstring 16 ch (2)");  
}

int
main ()
{
  int result;
  int j;

  serial_init ();
  test_depend ("serial:lcd");
  test_start ("panel-test");

  lock ();
  /* Reset the LCD controller.  */
  lcd_reset ();

  panel_initialize ();
  panel_create (&root_panel);
  panel_push (&root_panel);
  test_repeat = 10;
  for (j = 0; j < test_repeat; j++)
    {
      result = test_create_panels ();
      if (result != 0)
        break;

      result = test_panel_putchar ();
      if (result != 0)
        break;

      result = test_panel_putstring ();
      if (result != 0)
        break;

      result = test_panel_pushpop ();
      if (result != 0)
        break;
    }
  result = test_end ("panel-test");
  if (result == 0)
    {
      panel_benchs ();
    }
  return result;
}
